import cv2
import time
from .common import is_sitting, is_slouching
from .common import check_status
import socket
import struct
import threading
import datetime

# from .device import activate_buzzer
# import RPi.GPIO as GPIO
import os
import time
from ..communication import send_email

# import smbus


def get_path(parent_path):
    # 获取当前文件的绝对路径
    current_file_path = os.path.abspath(__file__)

    # 获取当前文件的父文件夹路径
    parent_parent_directory = os.path.dirname(os.path.dirname(current_file_path))

    # 构建指向父文件夹中的 mailPic 目录的路径
    paths = os.path.join(parent_parent_directory, parent_path)
    print(paths)
    return paths


def send_relax_signal(
    cap,
    path,
    pack_trans,
    server_email,
    server_password,
    smtp_server,
    smtp_port,
    target_email,
):
    output_path = os.path.join(path, "output_image.jpeg")
    success, img_output = cap.read()
    cv2.imwrite(output_path, img_output)
    if pack_trans:
        # 设置服务器的IP地址和端口号
        server_ip = "10.13.220.234"  # 替换X为服务器的实际IP地址
        server_port = 12345

        # 创建一个socket对象
        client_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        print(11234)
        # 连接到服务器
        client_socket.connect((server_ip, server_port))
        print(4322)

        # 创建一个8位的布尔数组
        bool_array = [1, 0, 0, 1, 0, 0, 1, 0]  # 示例数组

        # 将布尔数组打包成一个字节
        packed_data = struct.pack("B", int("".join(map(str, bool_array)), 2))

        # 发送数据到服务器
        client_socket.sendall(packed_data)
        with open("resources/test.png", "rb") as image_file:
            image_data = image_file.read()

        # 发送图片数据长度
        client_socket.sendall(len(image_data).to_bytes(4, byteorder="big"))

        # 发送图片数据
        client_socket.sendall(image_data)
    else:
        current_time = datetime.datetime.now().strftime("%Y年%m月%d日 %H:%M:%S")
        send_email(
            subject="国家反卷总局消息",
            body=f"<h1>来自 🤡🤡🤡🤡🤡</h1><p>国家反卷中心提示您，您应该休息了！！下载[国家反卷中心APP](https://sergiudm.github.io/detective/)，查看更多信息！</p>",
            to_emails=target_email,
            from_email=server_email,
            password=server_password,
            smtp_server=smtp_server,
            smtp_port=smtp_port,
            image_path=output_path,  # Use the first image found
        )
        # 发邮件
    os.remove(output_path)
    print("Thread finished")


def relax_detect(
    mpPose,
    pose,
    mpDraw,
    cap,
    image_path,
    protocol,
    pin,
    send_delay,
    effective_detection_duration,
    use_vis,
    pack_trans,
):
    # initial sensor pin
    # Pin_buzzer = pin
    # GPIO.setmode(GPIO.BCM)
    # GPIO.setup(Pin_buzzer, GPIO.OUT)
    path = get_path(image_path)
    server_email = protocol[0]
    server_password = protocol[1]
    smtp_server = protocol[2]
    smtp_port = int(protocol[3])
    target_email = protocol[4]
    pTime = 0
    try:
        model_1_time, model_1_state = 0, 0
        sitting_start_time = None
        slouching_start_time = None
        sitting = False
        slouching = False
        while True:
            success, img = cap.read()

            if not success:
                print("Error: Failed to read frame")
                break
            imgRGB = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
            results = pose.process(imgRGB)

            if results.pose_landmarks:
                # print("person detected!")
                mpDraw.draw_landmarks(
                    img, results.pose_landmarks, mpPose.POSE_CONNECTIONS
                )
                mpDraw.draw_landmarks(
                    img, results.pose_landmarks, mpPose.POSE_CONNECTIONS
                )
                for id, lm in enumerate(results.pose_landmarks.landmark):
                    h, w, c = img.shape
                    cx, cy = int(lm.x * w), int(lm.y * h)
                    cv2.circle(img, (cx, cy), 5, (255, 0, 0), cv2.FILLED)
                landmarks = results.pose_landmarks.landmark

                sitting, sitting_start_time = check_status(
                    is_sitting,
                    landmarks,
                    mpPose,
                    sitting_start_time,
                    effective_detection_duration,
                )
                slouching, slouching_start_time = check_status(
                    is_slouching,
                    landmarks,
                    mpPose,
                    slouching_start_time,
                    effective_detection_duration,
                )

                working = sitting and slouching

                if sitting and model_1_time == 0:  # FIXME: relace with working
                    model_1_time = time.time()
                    model_1_state = 1
                    detection_thread = threading.Thread(
                        target=send_relax_signal,
                        args=(
                            cap,
                            path,
                            pack_trans,
                            server_email,
                            server_password,
                            smtp_server,
                            smtp_port,
                            target_email,
                        ),
                    )
                    detection_thread.start()
                    print("Detection thread started")

                if time.time() - model_1_time > send_delay and model_1_state == 1:
                    model_1_state = 0
                    model_1_time = 0

                status_text = "Sitting" if sitting else "Not Sitting"
                j_text = ""
                if slouching:
                    j_text = "内卷！"

                cv2.putText(
                    img,
                    status_text,
                    (100, 100),
                    cv2.FONT_HERSHEY_SIMPLEX,
                    3,
                    (0, 255, 0) if sitting else (0, 0, 255),
                    3,
                )

            cTime = time.time()
            fps = 1.0 / (cTime - pTime)
            pTime = cTime
            cv2.putText(
                img, str(int(fps)), (70, 50), cv2.FONT_HERSHEY_PLAIN, 3, (255, 0, 0), 3
            )

            if cv2.waitKey(1) == ord("q"):
                break
            if use_vis:
                cv2.imshow("Image", img)
            cv2.waitKey(1)
    except KeyboardInterrupt:
        cap.release()
        cv2.destroyAllWindows()
        print("Exit")

    finally:
        cap.release()
        cv2.destroyAllWindows()



import socket
import struct

# 设置服务器的IP地址和端口号
server_ip = "10.13.220.234"  # 替换X为服务器的实际IP地址
server_port = 12345

# 创建一个socket对象
client_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

# 连接到服务器
client_socket.connect((server_ip, server_port))

try:
    # 创建一个8位的布尔数组
    bool_array = [1, 0, 0, 1, 0, 0, 1, 0]  # 示例数组

    # 将布尔数组打包成一个字节
    packed_data = struct.pack("B", int("".join(map(str, bool_array)), 2))
    # 发送数据到服务器
    client_socket.sendall(packed_data)

    with open("resources/test.png", "rb") as image_file:
        image_data = image_file.read()

    # 发送图片数据长度
    client_socket.sendall(len(image_data).to_bytes(4, byteorder="big"))

    # 发送图片数据
    client_socket.sendall(image_data)

    ###

    # 接收服务器的响应
    #response = client_socket.recv(1024)
    #print(f"Received from server: {response.decode('utf-8')}")
except KeyboardInterrupt:
    # shutdown the connection
    client_socket.shutdown(socket.SHUT_RDWR)
    client_socket.close()
    print("Exit")
finally:
    # 关闭连接
    client_socket.close()



import json
import os


class Config:
    def __init__(self, config_path="config.json"):
        self.config_path = config_path
        self.config_data = self.load_config()

    def modify_path(self, path):
        if os.name == "nt":
            print("running on Windows")
            json_file_path = json_file_path.replace("/", "\\")
            path = os.path.abspath(json_file_path)

        return path

    def load_config(self):
        if not os.path.exists(self.config_path):
            raise FileNotFoundError(f"Config file '{self.config_path}' not found.")
        with open(self.config_path, "r") as file:
            return json.load(file)

    def get_param(self, key, default=None):
        return self.config_data.get(key, default)

    def print_info(self):
        for key, value in self.config_data.items():
            print(f"{key}: {value}")



import time

import RPi.GPIO as GPIO

def activate_buzzer(pin, state=0):
    """
    Activates a buzzer connected to the specified GPIO pin for a given duration.

    :param pin: GPIO pin number where the buzzer is connected.
    :param duration: Duration in seconds for which the buzzer should sound.
    """
    # Set up the GPIO pin
    GPIO.setmode(GPIO.BCM)
    GPIO.setup(pin, GPIO.OUT)
    GPIO.output(pin, state)



import mediapipe as mp
import math
import time


def points_distance(x0, y0, x1, y1):
    return math.sqrt((x0 - x1) ** 2 + (y0 - y1) ** 2)


def calculate_angle(a, b, c):
    a = [a.x, a.y]
    b = [b.x, b.y]
    c = [c.x, c.y]

    ab = [a[0] - b[0], a[1] - b[1]]
    cb = [c[0] - b[0], c[1] - b[1]]

    dot_product = ab[0] * cb[0] + ab[1] * cb[1]
    ab_magnitude = math.hypot(ab[0], ab[1])
    cb_magnitude = math.hypot(cb[0], cb[1])

    angle = math.acos(dot_product / (ab_magnitude * cb_magnitude))
    return math.degrees(angle)


def calculate_angle2(x0, y0, x1, y1, x2, y2, x3, y3):
    AB = [x1 - x0, y1 - y0]
    CD = [x3 - x2, y3 - y2]

    dot_product = AB[0] * CD[0] + AB[1] * CD[1]

    AB_distance = points_distance(x0, y0, x1, y1) + 0.001  # 防止分母出现0
    CD_distance = points_distance(x2, y2, x3, y3) + 0.001

    cos_theta = dot_product / (AB_distance * CD_distance)

    theta = math.acos(cos_theta)

    return theta


def check_status(detect_function, landmarks, mpPose, start_time, threshold):
    status = detect_function(landmarks, mpPose=mpPose)
    if status:
        if start_time is None:
            start_time = time.time()
        elif time.time() - start_time >= threshold:
            return True, start_time
    else:
        start_time = None
    return False, start_time


def is_sitting(landmarks, mpPose):
    left_hip = landmarks[mpPose.PoseLandmark.LEFT_HIP.value]
    left_knee = landmarks[mpPose.PoseLandmark.LEFT_KNEE.value]
    left_ankle = landmarks[mpPose.PoseLandmark.LEFT_ANKLE.value]

    # Calculate angle at the left knee
    angle = calculate_angle(left_hip, left_knee, left_ankle)

    return angle < 130


def is_slouching(landmarks, mpPose):
    left_shoulder = landmarks[mpPose.PoseLandmark.LEFT_SHOULDER.value]
    left_hip = landmarks[mpPose.PoseLandmark.LEFT_HIP.value]
    left_knee = landmarks[mpPose.PoseLandmark.LEFT_KNEE.value]

    # Calculate angle at the left hip
    angle = calculate_angle(left_shoulder, left_hip, left_knee)

    # Assuming an angle less than 160 degrees indicates slouching
    return angle < 160


def detect_all_finger_state(all_points):
    finger_first_angle_bend_threshold = math.pi * 0.25  # 大拇指弯曲阈值
    finger_other_angle_bend_threshold = math.pi * 0.5  # 其他手指弯曲阈值
    finger_other_angle_straighten_threshold = math.pi * 0.2  # 其他手指伸直阈值

    first_is_bend = False
    first_is_straighten = False
    second_is_bend = False
    second_is_straighten = False
    third_is_bend = False
    third_is_straighten = False
    fourth_is_bend = False
    fourth_is_straighten = False
    fifth_is_bend = False
    fifth_is_straighten = False

    finger_first_angle = calculate_angle2(
        all_points["point0"][0],
        all_points["point0"][1],
        all_points["point1"][0],
        all_points["point1"][1],
        all_points["point2"][0],
        all_points["point2"][1],
        all_points["point4"][0],
        all_points["point4"][1],
    )
    finger_sencond_angle = calculate_angle2(
        all_points["point0"][0],
        all_points["point0"][1],
        all_points["point5"][0],
        all_points["point5"][1],
        all_points["point6"][0],
        all_points["point6"][1],
        all_points["point8"][0],
        all_points["point8"][1],
    )
    finger_third_angle = calculate_angle2(
        all_points["point0"][0],
        all_points["point0"][1],
        all_points["point9"][0],
        all_points["point9"][1],
        all_points["point10"][0],
        all_points["point10"][1],
        all_points["point12"][0],
        all_points["point12"][1],
    )
    finger_fourth_angle = calculate_angle2(
        all_points["point0"][0],
        all_points["point0"][1],
        all_points["point13"][0],
        all_points["point13"][1],
        all_points["point14"][0],
        all_points["point14"][1],
        all_points["point16"][0],
        all_points["point16"][1],
    )
    finger_fifth_angle = calculate_angle2(
        all_points["point0"][0],
        all_points["point0"][1],
        all_points["point17"][0],
        all_points["point17"][1],
        all_points["point18"][0],
        all_points["point18"][1],
        all_points["point20"][0],
        all_points["point20"][1],
    )

    if finger_first_angle > finger_first_angle_bend_threshold:  # 判断大拇指是否弯曲
        first_is_bend = True
        first_is_straighten = False
    else:
        first_is_bend = False
        first_is_straighten = True

    if finger_sencond_angle > finger_other_angle_bend_threshold:  # 判断食指是否弯曲
        second_is_bend = True
    elif finger_sencond_angle < finger_other_angle_straighten_threshold:
        second_is_straighten = True
    else:
        second_is_bend = False
        second_is_straighten = False

    if finger_third_angle > finger_other_angle_bend_threshold:  # 判断中指是否弯曲
        third_is_bend = True
    elif finger_third_angle < finger_other_angle_straighten_threshold:
        third_is_straighten = True
    else:
        third_is_bend = False
        third_is_straighten = False

    if finger_fourth_angle > finger_other_angle_bend_threshold:  # 判断无名指是否弯曲
        fourth_is_bend = True
    elif finger_fourth_angle < finger_other_angle_straighten_threshold:
        fourth_is_straighten = True
    else:
        fourth_is_bend = False
        fourth_is_straighten = False

    if finger_fifth_angle > finger_other_angle_bend_threshold:  # 判断小拇指是否弯曲
        fifth_is_bend = True
    elif finger_fifth_angle < finger_other_angle_straighten_threshold:
        fifth_is_straighten = True
    else:
        fifth_is_bend = False
        fifth_is_straighten = False

    # 将手指的弯曲或伸直状态存在字典中，简化后续函数的参数
    bend_states = {
        "first": first_is_bend,
        "second": second_is_bend,
        "third": third_is_bend,
        "fourth": fourth_is_bend,
        "fifth": fifth_is_bend,
    }
    straighten_states = {
        "first": first_is_straighten,
        "second": second_is_straighten,
        "third": third_is_straighten,
        "fourth": fourth_is_straighten,
        "fifth": fifth_is_straighten,
    }

    return bend_states, straighten_states


def detect_hand_state(all_points, bend_states, straighten_states):
    state_OK = judge_OK(all_points, bend_states, straighten_states)
    state_Return = judge_Return(all_points, bend_states, straighten_states)
    state_Left = judge_Left(all_points, bend_states, straighten_states)
    state_Right = judge_Right(all_points, bend_states, straighten_states)
    state_Like = judge_Like(all_points, bend_states, straighten_states)
    state_Pause = judge_Pause(all_points, bend_states, straighten_states)

    if state_OK == "OK":
        return "OK"
    elif state_Return == "Return":
        return "Return"
    elif state_Left == "Left":
        return "Left"
    elif state_Right == "Right":
        return "Right"
    elif state_Like == "Like":
        return "Like"
    elif state_Pause == "Pause":
        return "Pause"
    else:
        return "None"


def judge_OK(all_points, bend_states, straighten_states):
    angle5_6_and_6_8 = calculate_angle2(
        all_points["point5"][0],
        all_points["point5"][1],
        all_points["point6"][0],
        all_points["point6"][1],
        all_points["point6"][0],
        all_points["point6"][1],
        all_points["point8"][0],
        all_points["point8"][1],
    )

    if (
        angle5_6_and_6_8 > 0.1 * math.pi
        and straighten_states["third"]
        and straighten_states["fourth"]
        and straighten_states["fifth"]
    ):

        distance4_and_8 = points_distance(
            all_points["point4"][0],
            all_points["point4"][1],
            all_points["point8"][0],
            all_points["point8"][1],
        )
        distance2_and_6 = points_distance(
            all_points["point2"][0],
            all_points["point2"][1],
            all_points["point6"][0],
            all_points["point6"][1],
        )
        distance4_and_6 = points_distance(
            all_points["point4"][0],
            all_points["point4"][1],
            all_points["point6"][0],
            all_points["point6"][1],
        )

        if (
            distance4_and_8 < distance2_and_6
            and distance4_and_6 > distance4_and_8
            and all_points["point11"][1] < all_points["point10"][1]
        ):
            return "OK"
        else:
            return False
    else:
        return False


def judge_Like(all_points, bend_states, straighten_states):
    angle2_4_and_2_8 = calculate_angle2(
        all_points["point2"][0],
        all_points["point2"][1],
        all_points["point4"][0],
        all_points["point4"][1],
        all_points["point2"][0],
        all_points["point2"][1],
        all_points["point8"][0],
        all_points["point8"][1],
    )
    angle2_3_and_3_4 = calculate_angle2(
        all_points["point2"][0],
        all_points["point2"][1],
        all_points["point3"][0],
        all_points["point3"][1],
        all_points["point3"][0],
        all_points["point3"][1],
        all_points["point4"][0],
        all_points["point4"][1],
    )
    angle5_6_and_6_8 = calculate_angle2(
        all_points["point5"][0],
        all_points["point5"][1],
        all_points["point6"][0],
        all_points["point6"][1],
        all_points["point6"][0],
        all_points["point6"][1],
        all_points["point8"][0],
        all_points["point8"][1],
    )
    angle9_10_and_10_12 = calculate_angle2(
        all_points["point9"][0],
        all_points["point9"][1],
        all_points["point10"][0],
        all_points["point10"][1],
        all_points["point10"][0],
        all_points["point10"][1],
        all_points["point12"][0],
        all_points["point12"][1],
    )
    angle13_14_and_14_16 = calculate_angle2(
        all_points["point13"][0],
        all_points["point13"][1],
        all_points["point14"][0],
        all_points["point14"][1],
        all_points["point14"][0],
        all_points["point14"][1],
        all_points["point16"][0],
        all_points["point16"][1],
    )
    angle17_18_and_18_20 = calculate_angle2(
        all_points["point17"][0],
        all_points["point17"][1],
        all_points["point18"][0],
        all_points["point18"][1],
        all_points["point18"][0],
        all_points["point18"][1],
        all_points["point20"][0],
        all_points["point20"][1],
    )
    angle0_6_and_0_4 = calculate_angle2(
        all_points["point0"][0],
        all_points["point0"][1],
        all_points["point6"][0],
        all_points["point6"][1],
        all_points["point0"][0],
        all_points["point0"][1],
        all_points["point4"][0],
        all_points["point4"][1],
    )

    if (
        angle2_3_and_3_4 < 0.2 * math.pi
        and bend_states["second"]
        and bend_states["third"]
        and bend_states["fourth"]
        and bend_states["fifth"]
    ) or (
        angle2_3_and_3_4 < 0.2 * math.pi
        and angle5_6_and_6_8 > 0.2 * math.pi
        and angle9_10_and_10_12 > 0.2 * math.pi
        and angle13_14_and_14_16 > 0.2 * math.pi
        and angle17_18_and_18_20 > 0.2 * math.pi
    ):

        angle0_0__and_0_4 = calculate_angle2(
            all_points["point0"][0],
            all_points["point0"][1],
            all_points["point0"][0] + 10,
            all_points["point0"][1],
            all_points["point0"][0],
            all_points["point0"][1],
            all_points["point4"][0],
            all_points["point4"][1],
        )

        if (
            angle0_0__and_0_4 > 0.25 * math.pi
            and angle0_0__and_0_4 < 0.75 * math.pi
            and all_points["point3"][1] > all_points["point4"][1]
            and all_points["point5"][1] < all_points["point9"][1]
            and all_points["point9"][1] < all_points["point13"][1]
            and all_points["point13"][1] < all_points["point17"][1]
            and all_points["point2"][1] < all_points["point5"][1]
            and angle0_6_and_0_4 > 0.1 * math.pi
            and angle2_4_and_2_8 > 0.1 * math.pi
        ):
            return "Like"
        else:
            return False
    else:
        return False


def judge_Pause(all_points, bend_states, straighten_states):
    angle0_5_and_0_17 = calculate_angle2(
        all_points["point0"][0],
        all_points["point0"][1],
        all_points["point5"][0],
        all_points["point5"][1],
        all_points["point0"][0],
        all_points["point0"][1],
        all_points["point17"][0],
        all_points["point17"][1],
    )
    angle2_3_and_2_4 = calculate_angle2(
        all_points["point2"][0],
        all_points["point2"][1],
        all_points["point3"][0],
        all_points["point3"][1],
        all_points["point2"][0],
        all_points["point2"][1],
        all_points["point4"][0],
        all_points["point4"][1],
    )
    angle5_6_and_6_8 = calculate_angle2(
        all_points["point5"][0],
        all_points["point5"][1],
        all_points["point6"][0],
        all_points["point6"][1],
        all_points["point6"][0],
        all_points["point6"][1],
        all_points["point8"][0],
        all_points["point8"][1],
    )
    angle9_10_and_10_12 = calculate_angle2(
        all_points["point9"][0],
        all_points["point9"][1],
        all_points["point10"][0],
        all_points["point10"][1],
        all_points["point10"][0],
        all_points["point10"][1],
        all_points["point12"][0],
        all_points["point12"][1],
    )
    angle13_14_and_14_16 = calculate_angle2(
        all_points["point13"][0],
        all_points["point13"][1],
        all_points["point14"][0],
        all_points["point14"][1],
        all_points["point14"][0],
        all_points["point14"][1],
        all_points["point16"][0],
        all_points["point16"][1],
    )
    angle17_18_and_18_20 = calculate_angle2(
        all_points["point17"][0],
        all_points["point17"][1],
        all_points["point18"][0],
        all_points["point18"][1],
        all_points["point18"][0],
        all_points["point18"][1],
        all_points["point20"][0],
        all_points["point20"][1],
    )
    angle1_2_and_2_4 = calculate_angle2(
        all_points["point1"][0],
        all_points["point1"][1],
        all_points["point2"][0],
        all_points["point2"][1],
        all_points["point2"][0],
        all_points["point2"][1],
        all_points["point4"][0],
        all_points["point4"][1],
    )

    if (
        angle2_3_and_2_4 < 0.2 * math.pi
        and angle5_6_and_6_8 < 0.07 * math.pi
        and angle9_10_and_10_12 < 0.07 * math.pi
        and angle13_14_and_14_16 < 0.07 * math.pi
        and angle17_18_and_18_20 < 0.07 * math.pi
        and angle9_10_and_10_12 < 0.07 * math.pi
        and angle1_2_and_2_4 < 0.25 * math.pi
    ):
        if (
            angle0_5_and_0_17 > 0.1 * math.pi
            and all_points["point3"][1] > all_points["point4"][1]
            and all_points["point6"][1] > all_points["point8"][1]
            and all_points["point10"][1] > all_points["point12"][1]
        ):
            return "Pause"
        else:
            return False
    else:
        return False


def judge_Left(all_points, bend_states, straighten_states):
    angle5_6_and_6_8 = calculate_angle2(
        all_points["point5"][0],
        all_points["point5"][1],
        all_points["point6"][0],
        all_points["point6"][1],
        all_points["point6"][0],
        all_points["point6"][1],
        all_points["point8"][0],
        all_points["point8"][1],
    )
    angle9_10_and_10_12 = calculate_angle2(
        all_points["point9"][0],
        all_points["point9"][1],
        all_points["point10"][0],
        all_points["point10"][1],
        all_points["point10"][0],
        all_points["point10"][1],
        all_points["point12"][0],
        all_points["point12"][1],
    )
    angle13_14_and_14_16 = calculate_angle2(
        all_points["point13"][0],
        all_points["point13"][1],
        all_points["point14"][0],
        all_points["point14"][1],
        all_points["point14"][0],
        all_points["point14"][1],
        all_points["point16"][0],
        all_points["point16"][1],
    )
    angle17_18_and_18_20 = calculate_angle2(
        all_points["point17"][0],
        all_points["point17"][1],
        all_points["point18"][0],
        all_points["point18"][1],
        all_points["point18"][0],
        all_points["point18"][1],
        all_points["point20"][0],
        all_points["point20"][1],
    )
    angle0_6_and_0_4 = calculate_angle2(
        all_points["point0"][0],
        all_points["point0"][1],
        all_points["point6"][0],
        all_points["point6"][1],
        all_points["point0"][0],
        all_points["point0"][1],
        all_points["point4"][0],
        all_points["point4"][1],
    )
    angle0_5_and_0_17 = calculate_angle2(
        all_points["point0"][0],
        all_points["point0"][1],
        all_points["point5"][0],
        all_points["point5"][1],
        all_points["point0"][0],
        all_points["point0"][1],
        all_points["point17"][0],
        all_points["point17"][1],
    )

    if (
        straighten_states["first"]
        and bend_states["second"]
        and bend_states["third"]
        and bend_states["fourth"]
        and bend_states["fifth"]
    ) or (
        straighten_states["first"]
        and angle5_6_and_6_8 > 0.2 * math.pi
        and angle9_10_and_10_12 > 0.2 * math.pi
        and angle13_14_and_14_16 > 0.2 * math.pi
        and angle17_18_and_18_20 > 0.2 * math.pi
    ):

        angle0_0__and_0_4 = calculate_angle2(
            all_points["point0"][0],
            all_points["point0"][1],
            all_points["point0"][0] + 10,
            all_points["point0"][1],
            all_points["point0"][0],
            all_points["point0"][1],
            all_points["point4"][0],
            all_points["point4"][1],
        )

        if (
            angle0_5_and_0_17 > 0.15 * math.pi
            and angle0_0__and_0_4 > 0.7 * math.pi
            and all_points["point3"][0] < all_points["point2"][0]
            and angle0_6_and_0_4 > 0.1 * math.pi
            and all_points["point11"][1] > all_points["point10"][1]
            and all_points["point7"][1] > all_points["point6"][1]
            and all_points["point15"][1] > all_points["point14"][1]
            and all_points["point19"][1] > all_points["point18"][1]
        ):
            return "Left"
        else:
            return False
    else:
        return False


def judge_Right(all_points, bend_states, straighten_states):
    angle0_5_and_0_17 = calculate_angle2(
        all_points["point0"][0],
        all_points["point0"][1],
        all_points["point5"][0],
        all_points["point5"][1],
        all_points["point0"][0],
        all_points["point0"][1],
        all_points["point17"][0],
        all_points["point17"][1],
    )
    angle5_6_and_6_8 = calculate_angle2(
        all_points["point5"][0],
        all_points["point5"][1],
        all_points["point6"][0],
        all_points["point6"][1],
        all_points["point6"][0],
        all_points["point6"][1],
        all_points["point8"][0],
        all_points["point8"][1],
    )
    angle9_10_and_10_12 = calculate_angle2(
        all_points["point9"][0],
        all_points["point9"][1],
        all_points["point10"][0],
        all_points["point10"][1],
        all_points["point10"][0],
        all_points["point10"][1],
        all_points["point12"][0],
        all_points["point12"][1],
    )
    angle13_14_and_14_16 = calculate_angle2(
        all_points["point13"][0],
        all_points["point13"][1],
        all_points["point14"][0],
        all_points["point14"][1],
        all_points["point14"][0],
        all_points["point14"][1],
        all_points["point16"][0],
        all_points["point16"][1],
    )
    angle17_18_and_18_20 = calculate_angle2(
        all_points["point17"][0],
        all_points["point17"][1],
        all_points["point18"][0],
        all_points["point18"][1],
        all_points["point18"][0],
        all_points["point18"][1],
        all_points["point20"][0],
        all_points["point20"][1],
    )
    angle0_6_and_0_4 = calculate_angle2(
        all_points["point0"][0],
        all_points["point0"][1],
        all_points["point6"][0],
        all_points["point6"][1],
        all_points["point0"][0],
        all_points["point0"][1],
        all_points["point4"][0],
        all_points["point4"][1],
    )

    if (
        straighten_states["first"]
        and bend_states["second"]
        and bend_states["third"]
        and bend_states["fourth"]
        and bend_states["fifth"]
    ) or (
        straighten_states["first"]
        and angle5_6_and_6_8 > 0.2 * math.pi
        and angle9_10_and_10_12 > 0.2 * math.pi
        and angle13_14_and_14_16 > 0.2 * math.pi
        and angle17_18_and_18_20 > 0.2 * math.pi
    ):

        angle0_0__and_0_4 = calculate_angle2(
            all_points["point0"][0],
            all_points["point0"][1],
            all_points["point0"][0] + 10,
            all_points["point0"][1],
            all_points["point0"][0],
            all_points["point0"][1],
            all_points["point4"][0],
            all_points["point4"][1],
        )

        if (
            angle0_5_and_0_17 > 0.15 * math.pi
            and angle0_0__and_0_4 < 0.25 * math.pi
            and all_points["point3"][0] > all_points["point2"][0]
            and angle0_6_and_0_4 > 0.1 * math.pi
            and all_points["point11"][1] > all_points["point10"][1]
            and all_points["point7"][1] > all_points["point6"][1]
            and all_points["point15"][1] > all_points["point14"][1]
            and all_points["point19"][1] > all_points["point18"][1]
        ):
            return "Right"
        else:
            return False
    else:
        return False


def judge_Return(all_points, bend_states, straighten_states):
    angle18_6_and_18_18_ = calculate_angle2(
        all_points["point18"][0],
        all_points["point18"][1],
        all_points["point6"][0],
        all_points["point6"][1],
        all_points["point18"][0],
        all_points["point18"][1],
        all_points["point18"][0] + 10,
        all_points["point18"][1],
    )
    angle_6_18_and_6_6_ = calculate_angle2(
        all_points["point6"][0],
        all_points["point6"][1],
        all_points["point18"][0],
        all_points["point18"][1],
        all_points["point6"][0],
        all_points["point6"][1],
        all_points["point6"][0] + 10,
        all_points["point6"][1],
    )
    angle_0_2_and_0_17 = calculate_angle2(
        all_points["point0"][0],
        all_points["point0"][1],
        all_points["point2"][0],
        all_points["point2"][1],
        all_points["point0"][0],
        all_points["point0"][1],
        all_points["point17"][0],
        all_points["point17"][1],
    )

    if (
        bend_states["first"]
        and bend_states["second"]
        and bend_states["third"]
        and bend_states["fourth"]
        and bend_states["fifth"]
        and angle_0_2_and_0_17 > 0.15 * math.pi
        and all_points["point7"][1] > all_points["point6"][1]
        and all_points["point11"][1] > all_points["point10"][1]
        and all_points["point15"][1] > all_points["point14"][1]
        and all_points["point19"][1] > all_points["point18"][1]
    ):

        if angle18_6_and_18_18_ < 0.1 * math.pi or angle_6_18_and_6_6_ < 0.1 * math.pi:
            return "Return"
        else:
            return False
    else:
        return False



import cv2
import mediapipe as mp
from .common import detect_all_finger_state, detect_hand_state

mp_drawing = mp.solutions.drawing_utils
mp_hands = mp.solutions.hands
hands = mp_hands.Hands(
    static_image_mode=False,
    max_num_hands=2,
    min_detection_confidence=0.65,
    min_tracking_confidence=0.65,
)


def show_message(frame, message, position=(10, 30)):
    cv2.putText(
        frame,
        message,
        position,
        cv2.FONT_HERSHEY_SIMPLEX,
        1,
        (0, 255, 0),
        2,
        cv2.LINE_AA,
    )


recent_states = [""] * 20


def gesture_detect(cap):
    """detect hand gesture"""
    while True:
        ret, frame = cap.read()
        frame = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
        h, w = frame.shape[:2]
        frame = cv2.flip(frame, 1)  # flip the frame horizontally
        results = hands.process(frame)
        frame = cv2.cvtColor(frame, cv2.COLOR_RGB2BGR)
        if results.multi_handedness:
            for hand_label in results.multi_handedness:
                # print(hand_label)
                pass
        if results.multi_hand_landmarks:
            for hand_landmarks in results.multi_hand_landmarks:
                # print("hand_landmarks:", hand_landmarks)
                # landmarks visualization
                mp_drawing.draw_landmarks(
                    frame, hand_landmarks, mp_hands.HAND_CONNECTIONS
                )
                # gesture recognition
                lm = results.multi_hand_landmarks[0]
            lmHand = mp_hands.HandLandmark

            landmark_list = [
                [] for _ in range(6)
            ]  # landmark_list有 6 个子列表，分别存储着根节点坐标（0点）以及其它 5 根手指的关键点坐标

            for index, landmark in enumerate(lm.landmark):
                x = int(landmark.x * w)
                y = int(landmark.y * h)
                if index == lmHand.WRIST:
                    landmark_list[0].append((x, y))
                elif 1 <= index <= 4:
                    landmark_list[1].append((x, y))
                elif 5 <= index <= 8:
                    landmark_list[2].append((x, y))
                elif 9 <= index <= 12:
                    landmark_list[3].append((x, y))
                elif 13 <= index <= 16:
                    landmark_list[4].append((x, y))
                elif 17 <= index <= 20:
                    landmark_list[5].append((x, y))

            # 获取所有关节点的坐标
            point0 = landmark_list[0][0]
            point1, point2, point3, point4 = (
                landmark_list[1][0],
                landmark_list[1][1],
                landmark_list[1][2],
                landmark_list[1][3],
            )
            point5, point6, point7, point8 = (
                landmark_list[2][0],
                landmark_list[2][1],
                landmark_list[2][2],
                landmark_list[2][3],
            )
            point9, point10, point11, point12 = (
                landmark_list[3][0],
                landmark_list[3][1],
                landmark_list[3][2],
                landmark_list[3][3],
            )
            point13, point14, point15, point16 = (
                landmark_list[4][0],
                landmark_list[4][1],
                landmark_list[4][2],
                landmark_list[4][3],
            )
            point17, point18, point19, point20 = (
                landmark_list[5][0],
                landmark_list[5][1],
                landmark_list[5][2],
                landmark_list[5][3],
            )

            # 将所有关键点的坐标存储到一起，简化后续函数的参数
            all_points = {
                "point0": landmark_list[0][0],
                "point1": landmark_list[1][0],
                "point2": landmark_list[1][1],
                "point3": landmark_list[1][2],
                "point4": landmark_list[1][3],
                "point5": landmark_list[2][0],
                "point6": landmark_list[2][1],
                "point7": landmark_list[2][2],
                "point8": landmark_list[2][3],
                "point9": landmark_list[3][0],
                "point10": landmark_list[3][1],
                "point11": landmark_list[3][2],
                "point12": landmark_list[3][3],
                "point13": landmark_list[4][0],
                "point14": landmark_list[4][1],
                "point15": landmark_list[4][2],
                "point16": landmark_list[4][3],
                "point17": landmark_list[5][0],
                "point18": landmark_list[5][1],
                "point19": landmark_list[5][2],
                "point20": landmark_list[5][3],
            }

            # 调用函数，判断每根手指的弯曲或伸直状态
            bend_states, straighten_states = detect_all_finger_state(all_points)

            # 调用函数，检测当前手势
            current_state = detect_hand_state(
                all_points, bend_states, straighten_states
            )

            # 更新最近状态列表
            recent_states.pop(0)
            recent_states.append(current_state)

            # 检查列表中的所有状态是否相同
            if (
                len(set(recent_states)) == 1
            ):  # 如果连续30帧的手势状态都相同，则认为手势稳定，输出当前手势
                print("Detected consistent hand state:", recent_states[0])
                cv2.putText(
                    frame,
                    current_state,
                    (10, 70),
                    cv2.FONT_HERSHEY_SIMPLEX,
                    1,
                    (0, 0, 255),
                    2,
                    cv2.LINE_AA,
                )

            for (
                hand_landmarks
            ) in (
                results.multi_hand_landmarks
            ):  # results.multi_hand_landmarks是一个列表，只有一个元素，存储着21个关键点的xyz坐标。使用for循环遍历
                mp_drawing.draw_landmarks(
                    frame,
                    hand_landmarks,
                    mp_hands.HAND_CONNECTIONS,
                    mp_drawing.DrawingSpec(
                        color=(0, 255, 0), thickness=2, circle_radius=4
                    ),
                    mp_drawing.DrawingSpec(
                        color=(255, 0, 0), thickness=2, circle_radius=2
                    ),
                )

        cv2.imshow("MediaPipe Hands", frame)
        if cv2.waitKey(1) & 0xFF == 27:
            break
    cap.release()



import os
import smtplib
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.mime.image import MIMEImage

def send_email(subject, body, to_emails, from_email, password, smtp_server, smtp_port, image_path=None):
    # Create the email
    msg = MIMEMultipart('related')
    msg['From'] = from_email
    msg['To'] = ", ".join(to_emails)
    msg['Subject'] = subject

    # Create the HTML part
    html_body = f"""
    <html>
    <body>
        {body}
        <br>
        <img src="cid:image1">
    </body>
    </html>
    """
    msg.attach(MIMEText(html_body, 'html'))

    # Attach an image if provided
    if image_path:
        with open(image_path, 'rb') as img_file:
            img_data = img_file.read()
            image = MIMEImage(img_data, name=os.path.basename(image_path))
            image.add_header('Content-ID', '<image1>')
            msg.attach(image)

    try:
        # Connect to the SMTP server
        server = smtplib.SMTP(smtp_server, smtp_port)
        server.starttls()
        server.login(from_email, password)

        # Send the email
        server.sendmail(from_email, to_emails, msg.as_string())
        print("Email sent successfully!")

    except Exception as e:
        print(f"Failed to send email: {e}")

    finally:
        server.quit()

# # Example usage
# if __name__ == "__main__":
#     # Get the first image from the directory
#     # 获取当前文件的绝对路径
#     current_file_path = os.path.abspath(__file__)

#     # 获取当前文件的父文件夹路径
#     parent_parent_directory = os.path.dirname(os.path.dirname(current_file_path))

#     # 构建指向父文件夹中的 mailPic 目录的路径
#     image_directory = os.path.join(parent_parent_directory, "resources")
    
#     image_files = [f for f in os.listdir(image_directory) if f.lower().endswith(('.png', '.jpg', '.jpeg', '.gif'))]
    
#     if image_files:
#         first_image_path = os.path.join(image_directory, image_files[0])
#     else:
#         first_image_path = None
#         print("No image files found in the directory.")

#     send_email(
#         subject="Test Email",
#         body="<h1>This is a test email for 🤡🤡🤡🤡🤡</h1><p>With an image attached below.</p>",
#         to_emails=["2824174663@qq.com", "12212635@mail.sustech.edu.cn"],
#         from_email="2990973166@qq.com",
#         password="xfmhwdmoutajdhed",
#         smtp_server="smtp.qq.com",
#         smtp_port=587,
#         image_path=first_image_path  # Use the first image found
#     )


from .common import is_sitting
from .common import is_slouching
from .common import check_status
from .common import judge_Pause, judge_OK, judge_Like
from .common import detect_all_finger_state, detect_hand_state
from .detect_master import relax_detect
from .detect_others import working_detect
from .gesture import gesture_detect
from .utils import Config

__all__ = [
    "is_sitting",
    "is_slouching",
    "check_status",
    "relax_detect",
    "working_detect",
    "gesture_detect",
    "Config",
    "judge_Pause",
    "judge_OK",
    "judge_Like",
    "detect_all_finger_state",
    "detect_hand_state",
]



from setuptools import setup, find_packages

setup(
    name="detective_pi",
    version="0.1.5",  
    author="Sergiu Han",  
    author_email="sergiudm@outlook.com",  
    description="A versatile tool for daily use",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/sergiudm/detective", 
    packages=find_packages(include=["detective", "detective.*"]),
    install_requires=[
        "opencv-python",
        "opencv-python-headless",
        "mediapipe",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License", 
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.10",  
    entry_points={  
    },
)



import socket
import struct
import threading

# 设置服务器的IP地址和端口号
server_ip = "10.12.107.213"  # "10.13.220.234"
server_port = 12345

# 创建一个socket对象
server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

# 绑定IP地址和端口号
server_socket.bind((server_ip, server_port))

# 使socket开始监听传入连接
server_socket.listen(5)

print(f"Server listening on {server_ip}:{server_port}")


def handle_client(client_socket, addr):
    print(f"Connected by {addr}")
    try:
        # 接收一个字节的数据
        data = client_socket.recv(1)
        if not data:
            return

        # 解包数据
        bool_array = struct.unpack("B", data)[0]
        print(f"Received boolean array as integer: {bool_array}")
        print(f"Received boolean array as binary: {bin(bool_array)[2:].zfill(8)}")

        # 接收图片数据长度
        image_length_data = client_socket.recv(4)
        image_length = int.from_bytes(image_length_data, byteorder="big")
        print(f"Image length: {image_length}")

        # 接收图片数据
        image_data = b""
        while len(image_data) < image_length:
            packet = client_socket.recv(image_length - len(image_data))
            if not packet:
                break
            image_data += packet

        # 保存图片
        with open("received_image.png", "wb") as image_file:
            image_file.write(image_data)
        print("Image received and saved as received_image.png")

    finally:
        client_socket.close()


if __name__ == "__main__":
    while True:
        # 等待客户端连接
        client_socket, addr = server_socket.accept()
        # 为每个客户端连接创建一个新线程
        client_thread = threading.Thread(
            target=handle_client, args=(client_socket, addr)
        )
        client_thread.start()
